@@ -949,7 +949,7 @@ class PhotoExporter:
         # export live_photo .mov file?
         live_photo = bool(options.live_photo and self.photo.live_photo)
         overwrite = any([options.overwrite, options.update, options.force_update])
-        edited_version = options.edited or self.photo.shared
+        edited_version = bool(options.edited or self.photo.shared)
         # shared photos (in shared albums) show up as not having adjustments (not edited)
         # but Photos is unable to export the "original" as only a jpeg copy is shared in iCloud
         # so tell Photos to export the current version in this case
@@ -1076,6 +1076,7 @@ class PhotoExporter:
             ValueError if export_as_hardlink and convert_to_jpeg both True
         """
 
+        verbose = options.verbose or self._verbose
         if options.export_as_hardlink and options.convert_to_jpeg:
             raise ValueError(
                 "export_as_hardlink and convert_to_jpeg cannot both be True"
@@ -1165,6 +1166,9 @@ class PhotoExporter:
 
                 try:
                     fileutil.copy(src, dest_str)
+                    verbose(
+                        f"Exported {self._filename(self.photo.original_filename)} to {self._filepath(dest_str)}"
+                    )
                 except Exception as e:
                     raise ExportError(
                         f"Error copying file {src} to {dest_str}: {e} ({lineno(__file__)})"
