@@ -12,9 +12,10 @@ from .._constants import (
     _PHOTOS_5_VERSION,
     _PHOTOS_6_MODEL_VERSION,
     _PHOTOS_7_MODEL_VERSION,
+    _PHOTOS_8_MODEL_VERSION,
     _TESTED_DB_VERSIONS,
 )
-from ..utils import _open_sql_file
+from ..sqlite_utils import sqlite_open_ro
 
 __all__ = [
     "get_db_version",
@@ -36,7 +37,7 @@ def get_db_version(db_file):
 
     version = None
 
-    (conn, c) = _open_sql_file(db_file)
+    (conn, c) = sqlite_open_ro(db_file)
 
     # get database version
     c.execute("SELECT value from LiGlobals where LiGlobals.keyPath is 'libraryVersion'")
@@ -63,7 +64,7 @@ def get_model_version(db_file):
 
     version = None
 
-    (conn, c) = _open_sql_file(db_file)
+    (conn, c) = sqlite_open_ro(db_file)
 
     # get database version
     c.execute("SELECT MAX(Z_VERSION), Z_PLIST FROM Z_METADATA")
@@ -92,10 +93,12 @@ def get_db_model_version(db_file):
         return 6
     elif _PHOTOS_7_MODEL_VERSION[0] <= model_ver <= _PHOTOS_7_MODEL_VERSION[1]:
         return 7
+    elif _PHOTOS_8_MODEL_VERSION[0] <= model_ver <= _PHOTOS_8_MODEL_VERSION[1]:
+        return 8
     else:
         logging.warning(f"Unknown model version: {model_ver}")
         # cross our fingers and try latest version
-        return 7
+        return 8
 
 
 class UnknownLibraryVersion(Exception):
