@@ -2,7 +2,6 @@
 
 import datetime
 import fnmatch
-import glob
 import hashlib
 import importlib
 import inspect
@@ -12,16 +11,17 @@ import os.path
 import pathlib
 import platform
 import re
-import sqlite3
 import subprocess
 import sys
 import unicodedata
 import urllib.parse
 from plistlib import load as plistload
 from typing import Callable, List, Optional, Tuple, Union
+from uuid import UUID
 
 import CoreFoundation
 import requests
+import shortuuid
 
 from ._constants import UNICODE_FORMAT
 
@@ -41,6 +41,8 @@ __all__ = [
     "normalize_fs_path",
     "normalize_unicode",
     "pluralize",
+    "shortuuid_to_uuid",
+    "uuid_to_shortuuid",
 ]
 
 
@@ -359,44 +361,6 @@ def list_directory(
     return files
 
 
-def _open_sql_file(dbname):
-    """opens sqlite file dbname in read-only mode
-    returns tuple of (connection, cursor)"""
-    try:
-        dbpath = pathlib.Path(dbname).resolve()
-        conn = sqlite3.connect(f"{dbpath.as_uri()}?mode=ro", timeout=1, uri=True)
-        c = conn.cursor()
-    except sqlite3.Error as e:
-        sys.exit(f"An error occurred opening sqlite file: {e.args[0]} {dbname}")
-    return (conn, c)
-
-
-def _db_is_locked(dbname):
-    """check to see if a sqlite3 db is locked
-    returns True if database is locked, otherwise False
-    dbname: name of database to test"""
-
-    # first, check to see if lock file exists, if so, assume the file is locked
-    lock_name = f"{dbname}.lock"
-    if os.path.exists(lock_name):
-        logging.debug(f"{dbname} is locked")
-        return True
-
-    # no lock file so try to read from the database to see if it's locked
-    locked = None
-    try:
-        (conn, c) = _open_sql_file(dbname)
-        c.execute("SELECT name FROM sqlite_master WHERE type='table' ORDER BY name;")
-        conn.close()
-        logging.debug(f"{dbname} is not locked")
-        locked = False
-    except:
-        logging.debug(f"{dbname} is locked")
-        locked = True
-
-    return locked
-
-
 def normalize_unicode(value):
     """normalize unicode data"""
     if value is None:
@@ -534,3 +498,13 @@ def hexdigest(strval: str) -> str:
     h = hashlib.blake2b(digest_size=20)
     h.update(bytes(strval, "utf-8"))
     return h.hexdigest()
+
+
+def uuid_to_shortuuid(uuid: str) -> str:
+    """Convert uuid to shortuuid"""
+    return str(shortuuid.encode(UUID(uuid)))
+
+
+def shortuuid_to_uuid(short_uuid: str) -> str:
+    """Convert shortuuid to uuid"""
+    return str(shortuuid.decode(short_uuid)).upper()
